package learning.bigdata.mapreduce;

import java.io.IOException;

import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Mapper;

/**
 * This mapper is going to process each line in the input (simplified access log), find the country to which the IP
 * address belongs and emit the key-value pair in the form of <Country, 1>.
 * 
 * @author amar
 * 
 */
public class HitsByCountryMapper extends Mapper<LongWritable, Text, Text, IntWritable> {

	private final static String[] COUNTRIES = { "India", "UK", "US", "China" };
	private Text outputKey = new Text();
	private IntWritable outputValue = new IntWritable();

	@Override
	protected void setup(Context context) throws IOException, InterruptedException {

		super.setup(context);
	}

	@Override
	public void map(LongWritable key, Text value, Context context) throws IOException, InterruptedException {

		// 'value' is expected to have one line from input file
		// It is expected to be in following format:
		// Date, ipAddress
		try {
			String valueString = value.toString();

			// Split the value string to get Date and ipAddress
			String[] row = valueString.split(",");

			// row[0]= Date and row[1]=ipAddress
			String ipAddress = row[1];

			// Get the country name to which the ipAddress belongs
			String countryName = getCountryNameFromIpAddress(ipAddress);
			outputKey.set(countryName);
			outputValue.set(1);
			context.write(outputKey, outputValue);

		} catch (ArrayIndexOutOfBoundsException ex) {
			context.getCounter("Custom counters", "MAPPER_EXCEPTION_COUNTER").increment(1);
			ex.printStackTrace();
		}
	}

	/**
	 * This method is just for testing purpose and does not return the correct country for a given IP Address.
	 * It just returns one of the countries from { "India", "UK", "US", "China" }, on the basis of ipAddress.hashCode()
	 * 
	 * @param ipAddress
	 * @return The country name.
	 */
	private static String getCountryNameFromIpAddress(String ipAddress) {

		if (ipAddress != null && !ipAddress.isEmpty()) {

			int randomIndex = Math.abs(ipAddress.hashCode()) % COUNTRIES.length;
			return COUNTRIES[randomIndex];
		}

		return null;
	}
}